using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace Pluunty
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class PointerComponent<T> : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// Holds a reference to a SpriteBatch object used to render the pointer.
        /// </summary>
        protected internal SpriteBatch sb;
        /// <summary>
        /// Holds a collection of PointerItem types.
        /// </summary>
        protected internal Dictionary<T, PointerItem> pointers = new Dictionary<T, PointerItem>();
        /// <summary>
        /// Holds a reference to the current pointer key.
        /// </summary>
        protected internal T currentPointer;
        /// <summary>
        /// Holds a reference to the current pointer.
        /// </summary>
        protected internal PointerItem pointerVisual = null;
        /// <summary>
        /// Holds the current position of the pointer.
        /// </summary>
        protected internal Vector2 pos = Vector2.Zero;
        /// <summary>
        /// Used to store the minimum value that the pointer can move to.
        /// </summary>
        protected internal Vector2 min = new Vector2(float.MinValue, float.MinValue);
        /// <summary>
        /// Used to store the maximum value that the pointer can move to.
        /// </summary>
        protected internal Vector2 max = new Vector2(float.MaxValue, float.MaxValue);
        /// <summary>
        /// Holds a value that represents the depth of the pointer when drawing with the SpriteBatch object.
        /// </summary>
        protected internal float pointerDepth = 0;

        /// <summary>
        /// Gets/Sets a value that represents the depth of the pointer when drawing with the SpriteBatch object.
        /// </summary>
        public float PointerDepth
        {
            get { return pointerDepth; }
            set { pointerDepth = value; }
        }

        /// <summary>
        /// Gets>Sets the maximum value that the pointer can move to.
        /// </summary>
        public Vector2 Maximum
        {
            get { return max; }
            set
            {
                max = Vector2.Clamp(value, this.min, new Vector2(float.MaxValue, float.MaxValue));
                this.Position = this.pos;
            }
        }

        /// <summary>
        /// Gets>Sets the minimum value that the pointer can move to.
        /// </summary>
        public Vector2 Minimum
        {
            get { return min; }
            set
            {
                min = Vector2.Clamp(value, new Vector2(float.MinValue, float.MinValue), this.max);
                this.Position = this.pos;
            }
        }

        /// <summary>
        /// Gets/Sets the current position of the pointer.
        /// </summary>
        public Vector2 Position
        {
            get { return pos; }
            set { pos = Vector2.Clamp(value, this.min, this.max); }
        }

        /// <summary>
        /// Gets/Sets the current X position of the pointer.
        /// </summary>
        public float PointerX
        {
            get { return this.pos.X; }
            set { this.pos.X = MathHelper.Clamp(value, this.min.X, this.max.X); }
        }

        /// <summary>
        /// Gets/Sets the current Y position of the pointer.
        /// </summary>
        public float PointerY
        {
            get { return this.pos.Y; }
            set { this.pos.Y = MathHelper.Clamp(value, this.min.Y, this.max.Y); }
        }

        /// <summary>
        /// Gets a reference to the current pointer.
        /// </summary>
        /// <returns>Returns a reference to the current pointer.</returns>
        public PointerItem GetCurrentPointer()
        {
            return this.pointers[this.currentPointer];
        }

        /// <summary>
        /// Clears the current pointer.
        /// </summary>
        public void SetNoPointer()
        {
            this.pointerVisual = null;
            this.currentPointer = default(T);
        }

        /// <summary>
        /// Gets/Sets the current pointer key used to access the current pointer.
        /// </summary>
        public T CurrentPointer
        {
            get { return currentPointer; }
            set
            {
                if (this.pointers.TryGetValue(value, out this.pointerVisual))
                {
                    currentPointer = value;
                }
            }
        }

        /// <summary>
        /// Gets the collection of PointerItem objects.
        /// </summary>
        public Dictionary<T, PointerItem> Pointers
        {
            get { return pointers; }
            set { pointers = value; }
        }

        /// <summary>
        /// Initializes a new instance of the PointerComponent object.
        /// </summary>
        /// <param name="game"></param>
        public PointerComponent(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
            this.sb = new SpriteBatch(this.GraphicsDevice);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (this.pointerVisual == null) return;
            this.pointerVisual.Animation.Update(gameTime);
            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            if (this.pointerVisual == null) return;

            // TODO: Add your draw code here
            //Point tmp;
            //tmp.X = this.pos.X - this.pointerVisual.Hotspot.X;
            //tmp.Y = this.pos.Y - this.pointerVisual.Hotspot.Y;
            //Rectangle dr = new Rectangle(tmp.X, tmp.Y,
            //                             this.pointerVisual.SourceRectangle.Width,
            //                             this.pointerVisual.SourceRectangle.Height);

            zeeAni.SpriteFrame pv = this.pointerVisual.Animation.CurrentFrame;
            if (pv.Texture != null)
            {
                this.sb.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                this.sb.Draw(pv.Texture, this.pos, pv.SourceRectangle, pv.Color, pv.Rotation, pv.Origin, pv.Scale, pv.Effect, this.pointerDepth);
                this.sb.End();
            }

            base.Draw(gameTime);
        }

    }
}